--
-- pivotingaxle
-- Specialization class for John Deere 7280R
--
-- @author  Mario
-- @date  24/08/2010
--
-- Copyright (C) Mario, All Rights Reserved.

pivotingaxle = {};

function pivotingaxle.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function pivotingaxle:load(xmlFile)
	
	self.maxTranslationPrCycle = 0.00008;
	
	-- Front axle animation
	self.frontAxle = {};
	self.frontAxle.rotNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontAxle#rotNode"));
	
	-- Animated wheels
	self.numAnimatedWheels = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.animatedWheels#count"), 0);
	self.animatedWheel = {};
    for i=1, self.numAnimatedWheels do
        local objname = string.format("vehicle.animatedWheels.wheel" .. "%d", i);
		self.animatedWheel[i] = {};
        self.animatedWheel[i].rotNode = Utils.indexToObject(self.components, getXMLString(xmlFile, objname .. "#rotNode"));
		self.animatedWheel[i].wheel = Utils.indexToObject(self.components, getXMLString(xmlFile, objname .. "#wheelIndex"));
		self.animatedWheel[i].fenderAttach = Utils.indexToObject(self.components, getXMLString(xmlFile, objname .. "#fenderAttach"));
    end;

end;

function pivotingaxle:delete()

end;

function pivotingaxle:readStream(streamId, connection)
end;

function pivotingaxle:writeStream(streamId, connection)
end;

function pivotingaxle:readUpdateStream(streamId, timestamp, connection)
	
end;

function pivotingaxle:writeUpdateStream(streamId, connection, dirtyMask)
	
end;

function pivotingaxle:mouseEvent(posX, posY, isDown, isUp, button)
end;

function pivotingaxle:keyEvent(unicode, sym, modifier, isDown)
	
end;

function pivotingaxle:update(dt)
	
	if self.frontAxle.rotNode ~= nil and self.wheels[1].repr ~= nil and self.wheels[2].repr ~= nil then
		local ax, ay, az = getWorldTranslation(self.wheels[1].repr);
        local bx, by, bz = getWorldTranslation(self.wheels[2].repr);
		local x, y, z = worldDirectionToLocal(getParent(self.frontAxle.rotNode), bx-ax, by-ay, bz-az);

        setDirection(self.frontAxle.rotNode, x, y, z, 1, 0, 1);

		local x2,y2,z2 = getRotation(self.frontAxle.rotNode);

		--renderText(0.5, 0.65, 0.040, "x=" .. string.format((x2/3.14) * 180));
		--renderText(0.5, 0.75, 0.040, "y=" .. string.format((y2/3.14) * 180));
		--renderText(0.5, 0.85, 0.040, "z=" .. string.format((z2/3.14) * 180));

		setRotation(self.frontAxle.rotNode,x2 - 1.571,y2,z2 + 1.571);
	end;
	
	-- Making the visible wheels rotate
	if self.wheels[1].repr ~= nil and self.wheels[2].repr ~= nil then
		local x,y,z = getRotation(self.wheels[1].driveNode);
		local x2,y2,z2 = getRotation(self.wheels[1].repr);
		setRotation(self.animatedWheel[1].wheel, x,y,z);
		setRotation(self.animatedWheel[1].rotNode, x2,y2,z2);

		local x3,y3,z3 = getRotation(self.wheels[2].driveNode);
		local x4,y4,z4 = getRotation(self.wheels[2].repr);
		setRotation(self.animatedWheel[2].wheel, x3,y3,z3);
		setRotation(self.animatedWheel[2].rotNode, x4,y4,z4);
	end;
	
	-- Dynamic fenders
	if self.animatedWheel[1].fenderAttach ~= nil and self.animatedWheel[2].fenderAttach ~= nil then
		local x, y, z = getRotation(self.animatedWheel[1].rotNode);
		local x2, y2, z2 = getRotation(self.animatedWheel[2].rotNode);

		local fenderYRot = (y * 0.6 - self.maxTranslationPrCycle * dt);
		local fenderY2Rot = (y2 * 0.6 - self.maxTranslationPrCycle * dt);

		setRotation(self.animatedWheel[1].fenderAttach, x, fenderYRot, z);
		setRotation(self.animatedWheel[2].fenderAttach, x2, fenderY2Rot, z2);

		--local x3,y3,z3 = getRotation(self.animatedWheel[1].fenderAttach);
		--renderText(0.5, 0.65, 0.040, string.format(y3));

		--local x4,y4,z4 = getRotation(self.animatedWheel[2].fenderAttach);
		--renderText(0.5, 0.55, 0.040, string.format(y4));
	end;
	
end;

function pivotingaxle:draw()
	
end;

function pivotingaxle:validateAttacherJoint(implement, jointDesc, dt)
	return true;
end;

function pivotingaxle:onEnter()
	
end;

function pivotingaxle:startMotor()
	
end;

function pivotingaxle:onLeave()
	
end;
